@@ -447,6 +447,11 @@ func (c controller) postInstallConfigs(ctx context.Context) error {
 		return errors.Wrapf(err, "Timeout while waiting for cluster operators to be available")
 	}
 
+	err = utils.WaitForPredicateWithContext(ctx, WaitTimeout, GeneralWaitInterval, c.updateNodesLabels)
+	if err != nil {
+		c.log.Warn("Failed to label the nodes")
+	}
+
 	err = utils.WaitForPredicateWithContext(ctx, WaitTimeout, GeneralWaitInterval, c.addRouterCAToClusterCA)
 	if err != nil {
 		return errors.Wrapf(err, "Timeout while waiting router ca data")
@@ -466,7 +471,8 @@ func (c controller) postInstallConfigs(ctx context.Context) error {
 
 	// Wait for OLM operators
 	if err = c.waitForOLMOperators(ctx); err != nil {
-		return errors.Wrapf(err, "Error while initializing OLM operators")
+		// no need to send error in case olm failed as service should move to degraded in that case
+		c.log.WithError(err).Warn("Error while initializing OLM operators")
 	}
 
 	return nil
@@ -476,6 +482,23 @@ func (c controller) waitForOLMOperators(ctx context.Context) error {
 	var operators []models.MonitoredOperator
 	var err error
 
+	// In case the timeout occur, we have to update the pending OLM operators to failed state,
+	// so the assisted-service can update the cluster state to completed.
+	defer func() {
+		if err == nil {
+			return
+		}
+		updateFunc := func() bool {
+			if err = c.updatePendingOLMOperators(ctx); err != nil {
+				return KeepWaiting
+			}
+			return ExitWaiting
+		}
+		err = utils.WaitForPredicateWithContext(ctx, WaitTimeout, GeneralWaitInterval, updateFunc)
+		if err != nil {
+			c.log.WithError(err).Error("Timeout while waiting for some of the operators and not able to update its state")
+		}
+	}()
 	// Get the monitored operators:
 	err = utils.Retry(maxFetchAttempts, FetchRetryInterval, c.log, func() error {
 		operators, err = c.ic.GetClusterMonitoredOLMOperators(context.TODO(), c.ClusterID, c.OpenshiftVersion)
@@ -509,12 +532,8 @@ func (c controller) waitForOLMOperators(ctx context.Context) error {
 		return errors.Wrapf(err, "Failed to apply post manifests")
 	}
 
-	if err != c.waitForCSV(ctx, waitTimeout) {
-		// In case the timeout occur, we have to update the pending OLM operators to failed state,
-		// so the assisted-service can update the cluster state to completed.
-		if err = c.updatePendingOLMOperators(ctx); err != nil {
-			return errors.Errorf("Timeout while waiting for some of the operators and not able to update its state")
-		}
+	err = c.waitForCSV(ctx, waitTimeout)
+	if err != nil {
 		return errors.Wrapf(err, "Timeout while waiting for OLM operators be installed")
 	}
 
@@ -973,6 +992,62 @@ func (c controller) waitingForClusterOperators(ctx context.Context) error {
 	return utils.WaitForPredicateWithTimer(ctxWithTimeout, WaitTimeout, GeneralProgressUpdateInt, isClusterVersionAvailable)
 }
 
+func areNodeLabelsUpdated(node *v1.Node, nodeLabels string) bool {
+	nodeLabelsPresent := node.Labels
+	nodeLabelsRequired := make(map[string]string)
+	_ = json.Unmarshal([]byte(nodeLabels), &nodeLabelsRequired)
+
+	for key, value := range nodeLabelsRequired {
+		if val, found := nodeLabelsPresent[key]; !found || value != val {
+			return false
+		}
+	}
+
+	return true
+}
+
+func (c *controller) updateNodesLabels() bool {
+	ignoreStatuses := []string{models.HostStatusDisabled, models.HostStatusError}
+	ctxReq := utils.GenerateRequestContext()
+	log := utils.RequestIDLogger(ctxReq, c.log)
+
+	assistedNodesMap, err := c.ic.GetHosts(ctxReq, log, ignoreStatuses)
+	if err != nil {
+		log.WithError(err).Error("Failed to get node map from the assisted service")
+		return KeepWaiting
+	}
+
+	retry := false
+
+	for hostname, hostData := range assistedNodesMap {
+
+		if len(hostData.Host.NodeLabels) == 0 {
+			continue
+		}
+
+		node, err := c.kc.GetNode(hostname)
+		if err != nil {
+			log.WithError(err).Errorf("Failed to get node %s from k8s client", hostname)
+			retry = true
+			continue
+		} else if areNodeLabelsUpdated(node, hostData.Host.NodeLabels) {
+			continue
+		}
+
+		err = c.kc.PatchNodeLabels(node.Name, hostData.Host.NodeLabels)
+		if err != nil {
+			log.WithError(err).Errorf("Failed to patch node %s with node labels %s", node.Name, hostData.Host.NodeLabels)
+			retry = true
+		}
+	}
+
+	if retry {
+		return KeepWaiting
+	} else {
+		return ExitWaiting
+	}
+}
+
 func (c controller) sendCompleteInstallation(ctx context.Context, isSuccess bool, errorInfo string) {
 	c.log.Infof("Start complete installation step, with params success: %t, error info: %s", isSuccess, errorInfo)
 	_ = utils.WaitForPredicateWithContext(ctx, CompleteTimeout, GeneralProgressUpdateInt, func() bool {
