@@ -3,11 +3,15 @@ package installer
 import (
 	"context"
 	"fmt"
+	"net/http"
 	"path/filepath"
+	"strings"
 	"time"
 
 	"github.com/go-openapi/swag"
 	"github.com/google/uuid"
+	"github.com/openshift/assisted-installer/src/main/drymock"
+	"github.com/openshift/assisted-service/pkg/secretdump"
 	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 	"github.com/thoas/go-funk"
@@ -24,6 +28,10 @@ import (
 	"github.com/openshift/assisted-service/models"
 )
 
+// In dry run mode we prefer to get quick feedback about errors rather
+// than keep retrying many times.
+const dryRunMaximumInventoryClientRetries = 3
+
 const (
 	InstallDir                   = "/opt/install-dir"
 	KubeconfigPath               = "/opt/openshift/auth/kubeconfig"
@@ -50,14 +58,14 @@ type Installer interface {
 
 type installer struct {
 	config.Config
-	log             *logrus.Logger
+	log             logrus.FieldLogger
 	ops             ops.Ops
 	inventoryClient inventory_client.InventoryClient
 	kcBuilder       k8s_client.K8SClientBuilder
 	ign             ignition.Ignition
 }
 
-func NewAssistedInstaller(log *logrus.Logger, cfg config.Config, ops ops.Ops, ic inventory_client.InventoryClient, kcb k8s_client.K8SClientBuilder, ign ignition.Ignition) *installer {
+func NewAssistedInstaller(log logrus.FieldLogger, cfg config.Config, ops ops.Ops, ic inventory_client.InventoryClient, kcb k8s_client.K8SClientBuilder, ign ignition.Ignition) *installer {
 	return &installer{
 		log:             log,
 		Config:          cfg,
@@ -158,18 +166,43 @@ func (i *installer) InstallNode() error {
 	if err != nil {
 		i.log.Errorf("upload installation logs %s", err)
 	}
+	return i.finalize()
+}
+
+func (i *installer) finalize() error {
 	//update installation progress
 	i.UpdateHostInstallProgress(models.HostStageRebooting, "")
-	//reboot
-	if err = i.ops.Reboot(); err != nil {
+	if i.DryRunEnabled {
+		_, err := i.ops.ExecPrivilegeCommand(nil, "touch", i.Config.FakeRebootMarkerPath)
+		return errors.Wrap(err, "failed to touch fake reboot marker")
+	}
+
+	// in case ironic-agent exists on the host we should stop the assisted-agent service instead of rebooting the node.
+	// the assisted agent service stop will signal the ironic agent that we are done so that IPA can continue with its flow.
+	// regardless, we update the host install progress to `rebooting` since the node will get rebooted shortly after.
+	ironicAgentServiceName := "ironic-agent.service"
+	out, err := i.ops.ExecPrivilegeCommand(nil, "systemctl", "list-units", "--no-legend", ironicAgentServiceName)
+	if err != nil {
+		i.log.Errorf("Failed to check if ironic agent service exists on the node %s", err)
 		return err
 	}
+	if strings.Contains(out, ironicAgentServiceName) {
+		// in case the ironic-agent exists the installer should stop the assisted agent.service
+		if err = i.ops.SystemctlAction("stop", "agent.service"); err != nil {
+			return err
+		}
+	} else {
+		//reboot
+		if err = i.ops.Reboot(); err != nil {
+			return err
+		}
+	}
 	return nil
 }
 
 //updateSingleNodeIgnition will download the host ignition config and add the files under storage
 func (i *installer) updateSingleNodeIgnition(singleNodeIgnitionPath string) error {
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if i.DryRunEnabled {
 		return nil
 	}
 
@@ -291,7 +324,7 @@ func (i *installer) startBootstrap() error {
 }
 
 func (i *installer) extractIgnitionToFS(ignitionPath string) (err error) {
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if i.DryRunEnabled {
 		return nil
 	}
 
@@ -319,7 +352,7 @@ func (i *installer) extractIgnitionToFS(ignitionPath string) (err error) {
 }
 
 func (i *installer) generateSshKeyPair() error {
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if i.DryRunEnabled {
 		return nil
 	}
 
@@ -420,27 +453,33 @@ func numDone(hosts models.HostList) int {
 }
 
 func (i *installer) workerWaitFor2ReadyMasters(ctx context.Context) error {
+	var cluster *models.Cluster
+
 	i.log.Info("Waiting for 2 ready masters")
 	i.UpdateHostInstallProgress(models.HostStageWaitingForControlPlane, "")
-	cluster, err := i.inventoryClient.GetCluster(ctx, false)
-	if err != nil {
-		i.log.WithError(err).Errorf("Getting cluster %s", i.ClusterID)
-		return err
-	}
-	if swag.StringValue(cluster.Kind) == models.ClusterKindAddHostsCluster {
-		return nil
-	}
-	for {
-		hosts, err := i.inventoryClient.ListsHostsForRole(ctx, string(models.HostRoleMaster))
-		if err != nil {
-			i.log.WithError(err).Errorf("Getting cluster %s hosts", i.ClusterID)
-			return err
+	_ = utils.WaitForPredicate(waitForeverTimeout, generalWaitInterval, func() bool {
+		if cluster == nil {
+			var callErr error
+			cluster, callErr = i.inventoryClient.GetCluster(ctx, false)
+			if callErr != nil {
+				i.log.WithError(callErr).Errorf("Getting cluster %s", i.ClusterID)
+				return false
+			}
 		}
-		if numDone(hosts) >= minMasterNodes {
-			return nil
+		if swag.StringValue(cluster.Kind) == models.ClusterKindAddHostsCluster {
+			return true
 		}
-		time.Sleep(generalWaitInterval)
-	}
+
+		hosts, callErr := i.inventoryClient.ListsHostsForRole(ctx, string(models.HostRoleMaster))
+		if callErr != nil {
+			i.log.WithError(callErr).Errorf("Getting cluster %s hosts", i.ClusterID)
+			return false
+		}
+		return numDone(hosts) >= minMasterNodes
+
+	})
+
+	return nil
 }
 
 func (i *installer) shouldControlPlaneReplicasPatchApplied(kc k8s_client.K8SClient) (bool, error) {
@@ -691,11 +730,46 @@ func (i *installer) updateReadyMasters(nodes *v1.NodeList, readyMasters *[]strin
 }
 
 func (i *installer) cleanupInstallDevice() error {
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if i.DryRunEnabled {
 		return nil
 	}
 
-	vgName, err := i.ops.GetVGByPV(i.Device)
+	err := i.cleanupDevice(i.Device)
+
+	if err != nil {
+		return err
+	}
+
+	if i.ops.IsRaidMember(i.Device) {
+		i.log.Infof("A raid was detected on the device (%s) - cleaning", i.Device)
+		var devices []string
+		devices, err = i.ops.GetRaidDevices(i.Device)
+
+		if err != nil {
+			return err
+		}
+
+		for _, device := range devices {
+			// Cleaning the raid device itself before removing membership.
+			err = i.cleanupDevice(device)
+
+			if err != nil {
+				return err
+			}
+		}
+
+		err = i.ops.CleanRaidMembership(i.Device)
+
+		if err != nil {
+			return err
+		}
+	}
+
+	return i.ops.Wipefs(i.Device)
+}
+
+func (i *installer) cleanupDevice(device string) error {
+	vgName, err := i.ops.GetVGByPV(device)
 	if err != nil {
 		return err
 	}
@@ -708,23 +782,14 @@ func (i *installer) cleanupInstallDevice() error {
 			return err
 		}
 
-		err = i.ops.RemovePV(i.Device)
+		err = i.ops.RemovePV(device)
 
 		if err != nil {
 			return err
 		}
 	}
 
-	if i.ops.IsRaidDevice(i.Device) {
-		i.log.Infof("A raid was detected on the installation device (%s) - cleaning", i.Device)
-		err = i.ops.CleanRaidDevice(i.Device)
-
-		if err != nil {
-			return err
-		}
-	}
-
-	return i.ops.Wipefs(i.Device)
+	return nil
 }
 
 func (i *installer) verifyHostCanMoveToConfigurationStatus(inventoryHostsMapWithIp map[string]inventory_client.HostData) {
@@ -800,7 +865,7 @@ func (i *installer) createSingleNodeMasterIgnition() (string, error) {
 }
 
 func (i *installer) checkLocalhostName() error {
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if i.DryRunEnabled {
 		return nil
 	}
 
@@ -819,3 +884,57 @@ func (i *installer) checkLocalhostName() error {
 	i.log.Infof("write data into /etc/hostname")
 	return i.ops.CreateRandomHostname(data)
 }
+
+func RunInstaller(installerConfig *config.Config, logger logrus.FieldLogger) error {
+	logger.Infof("Assisted installer started. Configuration is:\n %s", secretdump.DumpSecretStruct(*installerConfig))
+	logger.Infof("Dry configuration is:\n %s", secretdump.DumpSecretStruct(installerConfig.DryRunConfig))
+
+	numRetries := inventory_client.DefaultMaxRetries
+	if installerConfig.DryRunEnabled {
+		numRetries = dryRunMaximumInventoryClientRetries
+	}
+
+	client, err := inventory_client.CreateInventoryClientWithDelay(
+		installerConfig.ClusterID,
+		installerConfig.URL,
+		installerConfig.PullSecretToken,
+		installerConfig.SkipCertVerification,
+		installerConfig.CACertPath,
+		logger,
+		http.ProxyFromEnvironment,
+		inventory_client.DefaultRetryMinDelay,
+		inventory_client.DefaultRetryMaxDelay,
+		numRetries,
+		inventory_client.DefaultMinRetries,
+	)
+
+	if err != nil {
+		logger.Fatalf("Failed to create inventory client %e", err)
+	}
+
+	o := ops.NewOpsWithConfig(installerConfig, logger, true)
+
+	var k8sClientBuilder k8s_client.K8SClientBuilder
+	if !installerConfig.DryRunEnabled {
+		k8sClientBuilder = k8s_client.NewK8SClient
+	} else {
+		k8sClientBuilder = drymock.NewDryRunK8SClientBuilder(installerConfig, o)
+	}
+
+	ai := NewAssistedInstaller(logger,
+		*installerConfig,
+		o,
+		client,
+		k8sClientBuilder,
+		ignition.NewIgnition(),
+	)
+
+	// Try to format requested disks. May fail formatting some disks, this is not an error.
+	ai.FormatDisks()
+
+	if err = ai.InstallNode(); err != nil {
+		ai.UpdateHostInstallProgress(models.HostStageFailed, err.Error())
+		return err
+	}
+	return nil
+}
