@@ -85,7 +85,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 
 	cleanInstallDevice := func() {
 		mockops.EXPECT().GetVGByPV(device).Return("", nil).Times(1)
-		mockops.EXPECT().IsRaidDevice(device).Return(false).Times(1)
+		mockops.EXPECT().IsRaidMember(device).Return(false).Times(1)
 		mockops.EXPECT().Wipefs(device).Return(nil).Times(1)
 	}
 
@@ -130,6 +130,9 @@ var _ = Describe("installer HostRoleMaster role", func() {
 	rebootSuccess := func() {
 		mockops.EXPECT().Reboot().Return(nil).Times(1)
 	}
+	ironicAgentDoesntExist := func() {
+		mockops.EXPECT().ExecPrivilegeCommand(gomock.Any(), "systemctl", "list-units", "--no-legend", "ironic-agent.service").Return("", nil).Times(1)
+	}
 	BeforeEach(func() {
 		ctrl = gomock.NewController(GinkgoT())
 		mockops = ops.NewMockOps(ctrl)
@@ -144,7 +147,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			"node1": {Host: &models.Host{InfraEnvID: nodesInfraEnvId, ID: &node1Id}, IPs: []string{"192.168.126.11"}},
 			"node2": {Host: &models.Host{InfraEnvID: nodesInfraEnvId, ID: &node2Id}, IPs: []string{"192.168.126.12"}}}
 	})
-	k8sBuilder := func(configPath string, logger *logrus.Logger) (k8s_client.K8SClient, error) {
+	k8sBuilder := func(configPath string, logger logrus.FieldLogger) (k8s_client.K8SClient, error) {
 		return mockk8sclient, nil
 	}
 
@@ -286,6 +289,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 					reportLogProgressSuccess()
 					setBootOrderSuccess(gomock.Any())
 					uploadLogsSuccess(true)
+					ironicAgentDoesntExist()
 					rebootSuccess()
 					ret := installerObj.InstallNode()
 					Expect(ret).Should(BeNil())
@@ -319,6 +323,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 					setBootOrderSuccess(gomock.Any())
 					uploadLogsSuccess(true)
 					reportLogProgressSuccess()
+					ironicAgentDoesntExist()
 					rebootSuccess()
 					ret := installerObj.InstallNode()
 					Expect(ret).Should(BeNil())
@@ -370,6 +375,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			setBootOrderSuccess(gomock.Any())
 			uploadLogsSuccess(true)
 			reportLogProgressSuccess()
+			ironicAgentDoesntExist()
 			rebootSuccess()
 			ret := installerObj.InstallNode()
 			Expect(ret).Should(BeNil())
@@ -497,6 +503,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 	})
 	Context("Master role", func() {
 		installerArgs := []string{"-n", "--append-karg", "nameserver=8.8.8.8"}
+		raidDevice := "/dev/md0"
 		conf := config.Config{Role: string(models.HostRoleMaster),
 			ClusterID:        "cluster-id",
 			InfraEnvID:       "infra-env-id",
@@ -524,6 +531,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			setBootOrderSuccess(gomock.Any())
 			uploadLogsSuccess(false)
 			reportLogProgressSuccess()
+			ironicAgentDoesntExist()
 			rebootSuccess()
 			ret := installerObj.InstallNode()
 			Expect(ret).Should(BeNil())
@@ -532,7 +540,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			cleanInstallDeviceClean := func() {
 				mockops.EXPECT().GetVGByPV(device).Return("vg1", nil).Times(1)
 				mockops.EXPECT().RemoveVG("vg1").Return(nil).Times(1)
-				mockops.EXPECT().IsRaidDevice(device).Return(false).Times(1)
+				mockops.EXPECT().IsRaidMember(device).Return(false).Times(1)
 				mockops.EXPECT().Wipefs(device).Return(nil).Times(1)
 				mockops.EXPECT().RemovePV(device).Return(nil).Times(1)
 			}
@@ -557,8 +565,10 @@ var _ = Describe("installer HostRoleMaster role", func() {
 		It("HostRoleMaster role raid cleanup disk - happy flow", func() {
 			cleanInstallDeviceClean := func() {
 				mockops.EXPECT().GetVGByPV(device).Return("", nil).Times(1)
-				mockops.EXPECT().IsRaidDevice(device).Return(true).Times(1)
-				mockops.EXPECT().CleanRaidDevice(device).Return(nil).Times(1)
+				mockops.EXPECT().IsRaidMember(device).Return(true).Times(1)
+				mockops.EXPECT().GetRaidDevices(device).Return([]string{raidDevice}, nil).Times(1)
+				mockops.EXPECT().GetVGByPV(raidDevice).Return("", nil).Times(1)
+				mockops.EXPECT().CleanRaidMembership(device).Return(nil).Times(1)
 				mockops.EXPECT().Wipefs(device).Return(nil).Times(1)
 			}
 			updateProgressSuccess([][]string{{string(models.HostStageStartingInstallation), conf.Role}})
@@ -573,14 +583,34 @@ var _ = Describe("installer HostRoleMaster role", func() {
 
 			cleanInstallDeviceClean := func() {
 				mockops.EXPECT().GetVGByPV(device).Return("", nil).Times(1)
-				mockops.EXPECT().IsRaidDevice(device).Return(true).Times(1)
-				mockops.EXPECT().CleanRaidDevice(device).Return(err).Times(1)
+				mockops.EXPECT().IsRaidMember(device).Return(true).Times(1)
+				mockops.EXPECT().GetRaidDevices(device).Return([]string{raidDevice}, nil).Times(1)
+				mockops.EXPECT().GetVGByPV(raidDevice).Return("", nil).Times(1)
+				mockops.EXPECT().CleanRaidMembership(device).Return(err).Times(1)
 			}
 			updateProgressSuccess([][]string{{string(models.HostStageStartingInstallation), conf.Role}})
 			cleanInstallDeviceClean()
 			ret := installerObj.InstallNode()
 			Expect(ret).Should(Equal(err))
 		})
+		It("master role happy flow with ironic agent", func() {
+			updateProgressSuccess([][]string{{string(models.HostStageStartingInstallation), conf.Role},
+				{string(models.HostStageInstalling), conf.Role},
+				{string(models.HostStageWritingImageToDisk)},
+				{string(models.HostStageRebooting)},
+			})
+			cleanInstallDevice()
+			mkdirSuccess(InstallDir)
+			downloadHostIgnitionSuccess(infraEnvId, hostId, "master-host-id.ign")
+			writeToDiskSuccess(installerArgs)
+			setBootOrderSuccess(gomock.Any())
+			uploadLogsSuccess(false)
+			reportLogProgressSuccess()
+			mockops.EXPECT().ExecPrivilegeCommand(gomock.Any(), "systemctl", "list-units", "--no-legend", "ironic-agent.service").Return("ironic-agent.service loaded active ", nil).Times(1)
+			mockops.EXPECT().SystemctlAction("stop", "agent.service").Return(nil).Times(1)
+			ret := installerObj.InstallNode()
+			Expect(ret).Should(BeNil())
+		})
 		It("HostRoleMaster role failed to create dir", func() {
 			updateProgressSuccess([][]string{{string(models.HostStageStartingInstallation), conf.Role}})
 			cleanInstallDevice()
@@ -626,6 +656,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			reportLogProgressSuccess()
 			writeToDiskSuccess(installerArgs)
 			setBootOrderSuccess(gomock.Any())
+			ironicAgentDoesntExist()
 			err := fmt.Errorf("failed to reboot")
 			mockops.EXPECT().Reboot().Return(err).Times(1)
 			ret := installerObj.InstallNode()
@@ -677,6 +708,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			// failure must do nothing
 			reportLogProgressSuccess()
 			mockops.EXPECT().UploadInstallationLogs(false).Return("", errors.Errorf("Dummy")).Times(1)
+			ironicAgentDoesntExist()
 			rebootSuccess()
 			ret := installerObj.InstallNode()
 			Expect(ret).Should(BeNil())
@@ -777,6 +809,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			setBootOrderSuccess(gomock.Any())
 			uploadLogsSuccess(true)
 			reportLogProgressSuccess()
+			ironicAgentDoesntExist()
 			rebootSuccess()
 			ret := installerObj.InstallNode()
 			Expect(ret).Should(BeNil())
