@@ -38,7 +38,6 @@ import (
 	runtimeclient "sigs.k8s.io/controller-runtime/pkg/client"
 	runtimeconfig "sigs.k8s.io/controller-runtime/pkg/client/config"
 
-	"github.com/openshift/assisted-installer/src/config"
 	"github.com/openshift/assisted-installer/src/ops"
 	"github.com/openshift/assisted-installer/src/utils"
 )
@@ -81,12 +80,14 @@ type K8SClient interface {
 	DeleteService(namespace, name string) error
 	DeletePods(namespace string) error
 	PatchNamespace(namespace string, data []byte) error
+	GetNode(name string) (*v1.Node, error)
+	PatchNodeLabels(nodeName string, nodeLabels string) error
 }
 
-type K8SClientBuilder func(configPath string, logger *logrus.Logger) (K8SClient, error)
+type K8SClientBuilder func(configPath string, logger logrus.FieldLogger) (K8SClient, error)
 
 type k8sClient struct {
-	log           *logrus.Logger
+	log           logrus.FieldLogger
 	client        *kubernetes.Clientset
 	ocClient      *operatorv1.Clientset
 	olmClient     *olmv1client.OperatorsV1alpha1Client
@@ -102,7 +103,7 @@ const (
 	CLUSTER_CONFIG_V1_NAME = "cluster-config-v1"
 )
 
-func NewK8SClient(configPath string, logger *logrus.Logger) (K8SClient, error) {
+func NewK8SClient(configPath string, logger logrus.FieldLogger) (K8SClient, error) {
 	config, err := clientcmd.BuildConfigFromFlags("", configPath)
 	if err != nil {
 		return &k8sClient{}, errors.Wrap(err, "loading kubeconfig")
@@ -204,10 +205,6 @@ func (c *k8sClient) ListMachines() (*machinev1beta1.MachineList, error) {
 }
 
 func (c *k8sClient) PatchEtcd() error {
-	if config.GlobalDryRunConfig.DryRunEnabled {
-		return nil
-	}
-
 	c.log.Info("Patching etcd")
 	data := []byte(`{"spec": {"unsupportedConfigOverrides": {"useUnsupportedUnsafeNonHANonProductionUnstableEtcd": true}}}`)
 	result, err := c.ocClient.OperatorV1().Etcds().Patch(context.Background(), "cluster", types.MergePatchType, data, metav1.PatchOptions{})
@@ -589,3 +586,17 @@ func (c *k8sClient) GetCSVFromSubscription(namespace string, name string) (strin
 	}
 	return result.Status.CurrentCSV, nil
 }
+
+func (c *k8sClient) GetNode(name string) (*v1.Node, error) {
+	node, err := c.client.CoreV1().Nodes().Get(context.TODO(), name, metav1.GetOptions{})
+	if err != nil {
+		return &v1.Node{}, err
+	}
+	return node, nil
+}
+
+func (c *k8sClient) PatchNodeLabels(nodeName string, nodeLabels string) error {
+	data := []byte(`{"metadata": {"labels": ` + nodeLabels + `}}`)
+	_, err := c.client.CoreV1().Nodes().Patch(context.Background(), nodeName, types.MergePatchType, data, metav1.PatchOptions{})
+	return err
+}
