@@ -46,8 +46,9 @@ type Ops interface {
 	RemoveLV(lvName, vgName string) error
 	RemovePV(pvName string) error
 	Wipefs(device string) error
-	IsRaidDevice(device string) bool
-	CleanRaidDevice(device string) error
+	IsRaidMember(device string) bool
+	GetRaidDevices(device string) ([]string, error)
+	CleanRaidMembership(device string) error
 	GetMCSLogs() (string, error)
 	UploadInstallationLogs(isBootstrap bool) (string, error)
 	ReloadHostFile(filepath string) error
@@ -73,26 +74,35 @@ const (
 )
 
 type ops struct {
-	log       *logrus.Logger
-	logWriter *utils.LogWriter
-	cmdEnv    []string
+	log             logrus.FieldLogger
+	logWriter       *utils.LogWriter
+	cmdEnv          []string
+	installerConfig *config.Config
+}
+
+func NewOps(logger *logrus.Logger, proxySet bool) Ops {
+	return NewOpsWithConfig(&config.Config{}, logger, proxySet)
 }
 
 // NewOps return a new ops interface
-func NewOps(logger *logrus.Logger, proxySet bool) Ops {
+func NewOpsWithConfig(installerConfig *config.Config, logger logrus.FieldLogger, proxySet bool) Ops {
 	cmdEnv := os.Environ()
-	if proxySet && (config.GlobalConfig.HTTPProxy != "" || config.GlobalConfig.HTTPSProxy != "") {
-		if config.GlobalConfig.HTTPProxy != "" {
-			cmdEnv = append(cmdEnv, fmt.Sprintf("HTTP_PROXY=%s", config.GlobalConfig.HTTPProxy))
+	if proxySet && (installerConfig.HTTPProxy != "" || installerConfig.HTTPSProxy != "") {
+		if installerConfig.HTTPProxy != "" {
+			cmdEnv = append(cmdEnv, fmt.Sprintf("HTTP_PROXY=%s", installerConfig.HTTPProxy))
 		}
-		if config.GlobalConfig.HTTPSProxy != "" {
-			cmdEnv = append(cmdEnv, fmt.Sprintf("HTTPS_PROXY=%s", config.GlobalConfig.HTTPSProxy))
+		if installerConfig.HTTPSProxy != "" {
+			cmdEnv = append(cmdEnv, fmt.Sprintf("HTTPS_PROXY=%s", installerConfig.HTTPSProxy))
 		}
-		if config.GlobalConfig.NoProxy != "" {
-			cmdEnv = append(cmdEnv, fmt.Sprintf("NO_PROXY=%s", config.GlobalConfig.NoProxy))
+		if installerConfig.NoProxy != "" {
+			cmdEnv = append(cmdEnv, fmt.Sprintf("NO_PROXY=%s", installerConfig.NoProxy))
 		}
 	}
-	return &ops{logger, utils.NewLogWriter(logger), cmdEnv}
+	return &ops{
+		log:             logger,
+		logWriter:       utils.NewLogWriter(logger),
+		cmdEnv:          cmdEnv,
+		installerConfig: installerConfig}
 }
 
 // ExecPrivilegeCommand execute a command in the host environment via nsenter
@@ -125,20 +135,21 @@ func (o *ops) ExecPrivilegeCommand(liveLogger io.Writer, command string, args ..
 }
 
 type ExecCommandError struct {
-	Command    string
-	Args       []string
-	Env        []string
-	ExitErr    error
-	Output     string
-	WaitStatus int
+	Command         string
+	Args            []string
+	Env             []string
+	ExitErr         error
+	Output          string
+	WaitStatus      int
+	PullSecretToken string
 }
 
-func removePullSecret(s []string) []string {
-	if config.GlobalConfig.PullSecretToken == "" {
+func removePullSecret(s []string, pullSecretToken string) []string {
+	if pullSecretToken == "" {
 		return s
 	}
 
-	return strings.Split(strings.ReplaceAll(strings.Join(s, " "), config.GlobalConfig.PullSecretToken, "<SECRET>"), " ")
+	return strings.Split(strings.ReplaceAll(strings.Join(s, " "), pullSecretToken, "<SECRET>"), " ")
 }
 
 func (e *ExecCommandError) Error() string {
@@ -146,11 +157,11 @@ func (e *ExecCommandError) Error() string {
 	if len(e.Output) > 200 {
 		lastOutput = "... " + e.Output[len(e.Output)-200:]
 	}
-	return fmt.Sprintf("failed executing %s %v, Error %s, LastOutput \"%s\"", e.Command, removePullSecret(e.Args), e.ExitErr, lastOutput)
+	return fmt.Sprintf("failed executing %s %v, Error %s, LastOutput \"%s\"", e.Command, removePullSecret(e.Args, e.PullSecretToken), e.ExitErr, lastOutput)
 }
 
 func (e *ExecCommandError) DetailedError() string {
-	return fmt.Sprintf("failed executing %s %v, env vars %v, error %s, waitStatus %d, Output \"%s\"", e.Command, removePullSecret(e.Args), removePullSecret(e.Env), e.ExitErr, e.WaitStatus, e.Output)
+	return fmt.Sprintf("failed executing %s %v, env vars %v, error %s, waitStatus %d, Output \"%s\"", e.Command, removePullSecret(e.Args, e.PullSecretToken), removePullSecret(e.Env, e.PullSecretToken), e.ExitErr, e.WaitStatus, e.Output)
 }
 
 // ExecCommand executes command.
@@ -178,11 +189,12 @@ func (o *ops) ExecCommand(liveLogger io.Writer, command string, args ...string)
 		}
 
 		execErr := &ExecCommandError{
-			Command: command,
-			Args:    args,
-			Env:     cmd.Env,
-			ExitErr: err,
-			Output:  output,
+			Command:         command,
+			Args:            args,
+			Env:             cmd.Env,
+			ExitErr:         err,
+			Output:          output,
+			PullSecretToken: o.installerConfig.PullSecretToken,
 		}
 		if exitErr, ok := err.(*exec.ExitError); ok {
 			if status, ok := exitErr.Sys().(syscall.WaitStatus); ok {
@@ -195,7 +207,8 @@ func (o *ops) ExecCommand(liveLogger io.Writer, command string, args ...string)
 		}
 		return output, execErr
 	}
-	o.log.Debug("Command executed:", " command", command, " arguments", removePullSecret(args), "env vars", removePullSecret(cmd.Env), "output", output)
+	o.log.Debug("Command executed:", " command", command, " arguments", removePullSecret(args, o.installerConfig.PullSecretToken), "env vars",
+		removePullSecret(cmd.Env, o.installerConfig.PullSecretToken), "output", output)
 	return output, err
 }
 
@@ -206,7 +219,7 @@ func (o *ops) Mkdir(dirName string) error {
 }
 
 func (o *ops) SystemctlAction(action string, args ...string) error {
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if o.installerConfig.DryRunEnabled {
 		return nil
 	}
 
@@ -223,14 +236,14 @@ func (o *ops) WriteImageToDisk(ignitionPath string, device string, progressRepor
 	o.log.Infof("Writing image and ignition to disk with arguments: %v", allArgs)
 
 	installerExecutable := coreosInstallerExecutable
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if o.installerConfig.DryRunEnabled {
 		// In dry run, we use an executable called dry-installer rather than coreos-installer.
 		// This executable is expected to pretend to be doing coreos-installer stuff and print fake
 		// progress. It's up to the dry-mode user to make sure such executable is available in PATH
 		installerExecutable = dryRunCoreosInstallerExecutable
 	}
 
-	_, err := o.ExecPrivilegeCommand(NewCoreosInstallerLogWriter(o.log, progressReporter, config.GlobalConfig.InfraEnvID, config.GlobalConfig.HostID),
+	_, err := o.ExecPrivilegeCommand(NewCoreosInstallerLogWriter(o.log, progressReporter, o.installerConfig.InfraEnvID, o.installerConfig.HostID),
 		installerExecutable, allArgs...)
 	return err
 }
@@ -254,7 +267,7 @@ func (o *ops) EvaluateDiskSymlink(device string) string {
 }
 
 func (o *ops) FormatDisk(disk string) error {
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if o.installerConfig.DryRunEnabled {
 		return nil
 	}
 
@@ -276,11 +289,6 @@ func installerArgs(ignitionPath string, device string, extra []string) []string
 }
 
 func (o *ops) Reboot() error {
-	if config.GlobalDryRunConfig.DryRunEnabled {
-		_, err := o.ExecPrivilegeCommand(o.logWriter, "touch", config.GlobalDryRunConfig.FakeRebootMarkerPath)
-		return errors.Wrap(err, "failed to touch fake reboot marker")
-	}
-
 	o.log.Info("Rebooting node")
 	_, err := o.ExecPrivilegeCommand(o.logWriter, "shutdown", "-r", "+1", "'Installation completed, server is going to reboot.'")
 	if err != nil {
@@ -291,7 +299,7 @@ func (o *ops) Reboot() error {
 }
 
 func (o *ops) SetBootOrder(device string) error {
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if o.installerConfig.DryRunEnabled {
 		return nil
 	}
 
@@ -325,7 +333,7 @@ func (o *ops) getEfiFilePath() string {
 }
 
 func (o *ops) ExtractFromIgnition(ignitionPath string, fileToExtract string) error {
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if o.installerConfig.DryRunEnabled {
 		return nil
 	}
 
@@ -366,7 +374,10 @@ func (o *ops) ExtractFromIgnition(ignitionPath string, fileToExtract string) err
 }
 
 func (o *ops) PrepareController() error {
-
+	// Do not prepare controller files in dry mode
+	if o.installerConfig.DryRunEnabled {
+		return nil
+	}
 	if err := o.renderControllerCm(); err != nil {
 		return err
 	}
@@ -397,13 +408,13 @@ func (o *ops) PrepareController() error {
 
 func (o *ops) renderControllerCm() error {
 	var params = map[string]interface{}{
-		"InventoryUrl":         config.GlobalConfig.URL,
-		"ClusterId":            config.GlobalConfig.ClusterID,
-		"SkipCertVerification": strconv.FormatBool(config.GlobalConfig.SkipCertVerification),
-		"CACertPath":           config.GlobalConfig.CACertPath,
-		"HaMode":               config.GlobalConfig.HighAvailabilityMode,
-		"CheckCVO":             config.GlobalConfig.CheckClusterVersion,
-		"MustGatherImage":      config.GlobalConfig.MustGatherImage,
+		"InventoryUrl":         o.installerConfig.URL,
+		"ClusterId":            o.installerConfig.ClusterID,
+		"SkipCertVerification": strconv.FormatBool(o.installerConfig.SkipCertVerification),
+		"CACertPath":           o.installerConfig.CACertPath,
+		"HaMode":               o.installerConfig.HighAvailabilityMode,
+		"CheckCVO":             o.installerConfig.CheckClusterVersion,
+		"MustGatherImage":      o.installerConfig.MustGatherImage,
 	}
 
 	return o.renderDeploymentFiles(filepath.Join(controllerDeployFolder, controllerDeployCmTemplate),
@@ -412,7 +423,7 @@ func (o *ops) renderControllerCm() error {
 
 func (o *ops) renderControllerSecret() error {
 	var params = map[string]interface{}{
-		"PullSecretToken": config.GlobalConfig.PullSecretToken,
+		"PullSecretToken": o.installerConfig.PullSecretToken,
 	}
 
 	return o.renderDeploymentFiles(filepath.Join(controllerDeployFolder, controllerDeploySecretTemplate),
@@ -421,13 +432,13 @@ func (o *ops) renderControllerSecret() error {
 
 func (o *ops) renderControllerPod() error {
 	var params = map[string]interface{}{
-		"ControllerImage":  config.GlobalConfig.ControllerImage,
-		"CACertPath":       config.GlobalConfig.CACertPath,
-		"OpenshiftVersion": config.GlobalConfig.OpenshiftVersion,
+		"ControllerImage":  o.installerConfig.ControllerImage,
+		"CACertPath":       o.installerConfig.CACertPath,
+		"OpenshiftVersion": o.installerConfig.OpenshiftVersion,
 	}
 
-	if config.GlobalConfig.ServiceIPs != "" {
-		params["ServiceIPs"] = strings.Split(config.GlobalConfig.ServiceIPs, ",")
+	if o.installerConfig.ServiceIPs != "" {
+		params["ServiceIPs"] = strings.Split(o.installerConfig.ServiceIPs, ",")
 	}
 
 	return o.renderDeploymentFiles(filepath.Join(controllerDeployFolder, controllerDeployPodTemplate),
@@ -518,8 +529,8 @@ func (o *ops) Wipefs(device string) error {
 	return err
 }
 
-func (o *ops) IsRaidDevice(device string) bool {
-	raidDevices, err := o.getRaidDevices()
+func (o *ops) IsRaidMember(device string) bool {
+	raidDevices, err := o.getRaidDevices2Members()
 
 	if err != nil {
 		o.log.WithError(err).Errorf("Error occurred while trying to get list of raid devices - continue without cleaning")
@@ -540,15 +551,15 @@ func (o *ops) IsRaidDevice(device string) bool {
 	return false
 }
 
-func (o *ops) CleanRaidDevice(device string) error {
-	raidDevices, err := o.getRaidDevices()
+func (o *ops) CleanRaidMembership(device string) error {
+	raidDevices, err := o.getRaidDevices2Members()
 
 	if err != nil {
 		return err
 	}
 
 	for raidDeviceName, raidArrayMembers := range raidDevices {
-		err = o.cleanDeviceFromRaidDevice(device, raidDeviceName, raidArrayMembers)
+		err = o.removeDeviceFromRaidArray(device, raidDeviceName, raidArrayMembers)
 
 		if err != nil {
 			return err
@@ -558,7 +569,30 @@ func (o *ops) CleanRaidDevice(device string) error {
 	return nil
 }
 
-func (o *ops) getRaidDevices() (map[string][]string, error) {
+func (o *ops) GetRaidDevices(deviceName string) ([]string, error) {
+	raidDevices, err := o.getRaidDevices2Members()
+	var result []string
+
+	if err != nil {
+		return result, err
+	}
+
+	for raidDeviceName, raidArrayMembers := range raidDevices {
+		expression, _ := regexp.Compile(deviceName + "[\\d]*")
+
+		for _, raidMember := range raidArrayMembers {
+			// A partition or the device itself is part of the raid array.
+			if expression.MatchString(raidMember) {
+				result = append(result, raidDeviceName)
+				break
+			}
+		}
+	}
+
+	return result, nil
+}
+
+func (o *ops) getRaidDevices2Members() (map[string][]string, error) {
 	output, err := o.ExecPrivilegeCommand(o.logWriter, "mdadm", "-v", "--query", "--detail", "--scan")
 
 	if err != nil {
@@ -605,7 +639,7 @@ func (o *ops) getRaidDevices() (map[string][]string, error) {
 	return result, nil
 }
 
-func (o *ops) cleanDeviceFromRaidDevice(deviceName string, raidDeviceName string, raidArrayMembers []string) error {
+func (o *ops) removeDeviceFromRaidArray(deviceName string, raidDeviceName string, raidArrayMembers []string) error {
 	raidStopped := false
 
 	expression, _ := regexp.Compile(deviceName + "[\\d]*")
@@ -638,9 +672,9 @@ func (o *ops) cleanDeviceFromRaidDevice(deviceName string, raidDeviceName string
 }
 
 func (o *ops) GetMCSLogs() (string, error) {
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if o.installerConfig.DryRunEnabled {
 		mcsLogs := ""
-		for _, clusterHost := range config.GlobalDryRunConfig.ParsedClusterHosts {
+		for _, clusterHost := range o.installerConfig.ParsedClusterHosts {
 			// Add IP access log for each IP, this is how the installer determines which node has downloaded the ignition
 			if !o.DryRebootHappened(clusterHost.RebootMarkerPath) {
 				// Host didn't even reboot yet, don't pretend it fetched the ignition
@@ -678,16 +712,16 @@ func (o *ops) GetMCSLogs() (string, error) {
 func (o *ops) UploadInstallationLogs(isBootstrap bool) (string, error) {
 	command := "podman"
 	args := []string{"run", "--rm", "--privileged", "--net=host", "--pid=host", "-v", "/run/systemd/journal/socket:/run/systemd/journal/socket",
-		"-v", "/var/log:/var/log", config.GlobalConfig.AgentImage, "logs_sender",
-		"-cluster-id", config.GlobalConfig.ClusterID, "-url", config.GlobalConfig.URL,
-		"-host-id", config.GlobalConfig.HostID, "-infra-env-id", config.GlobalConfig.InfraEnvID,
-		"-pull-secret-token", config.GlobalConfig.PullSecretToken,
-		fmt.Sprintf("-insecure=%s", strconv.FormatBool(config.GlobalConfig.SkipCertVerification)),
+		"-v", "/var/log:/var/log", o.installerConfig.AgentImage, "logs_sender",
+		"-cluster-id", o.installerConfig.ClusterID, "-url", o.installerConfig.URL,
+		"-host-id", o.installerConfig.HostID, "-infra-env-id", o.installerConfig.InfraEnvID,
+		"-pull-secret-token", o.installerConfig.PullSecretToken,
+		fmt.Sprintf("-insecure=%s", strconv.FormatBool(o.installerConfig.SkipCertVerification)),
 		fmt.Sprintf("-bootstrap=%s", strconv.FormatBool(isBootstrap)),
 	}
 
-	if config.GlobalConfig.CACertPath != "" {
-		args = append(args, fmt.Sprintf("-cacert=%s", config.GlobalConfig.CACertPath))
+	if o.installerConfig.CACertPath != "" {
+		args = append(args, fmt.Sprintf("-cacert=%s", o.installerConfig.CACertPath))
 	}
 	return o.ExecPrivilegeCommand(o.logWriter, command, args...)
 }
@@ -696,7 +730,7 @@ func (o *ops) UploadInstallationLogs(isBootstrap bool) (string, error) {
 // For example /etc/resolv.conf, it can't be changed with Z flag but is updated by bootkube.sh
 // and we need this update for dns resolve of kubeapi
 func (o *ops) ReloadHostFile(filepath string) error {
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if o.installerConfig.DryRunEnabled {
 		return nil
 	}
 
@@ -723,7 +757,7 @@ func (o *ops) ReloadHostFile(filepath string) error {
 }
 
 func (o *ops) CreateOpenshiftSshManifest(filePath, tmpl, sshPubKeyPath string) error {
-	if config.GlobalDryRunConfig.DryRunEnabled {
+	if o.installerConfig.DryRunEnabled {
 		return nil
 	}
 
